@@ -1,9 +1,11 @@
 package version::vpp;
 use strict;
 
+use POSIX qw/locale_h/;
 use locale;
 use vars qw ($VERSION @ISA @REGEXS);
-$VERSION = 0.7501;
+$VERSION = '0.77';
+$VERSION = eval $VERSION;
 
 push @REGEXS, qr/
 	^v?	# optional leading 'v'
@@ -37,7 +39,7 @@ sub new
 	my ($class, $value) = @_;
 	my $self = bless ({}, ref ($class) || $class);
 	
-	if ( ref($value) && eval("$value->isa('version')") ) {
+	if ( ref($value) && eval('$value->isa("version")') ) {
 	    # Can copy the elements directly
 	    $self->{version} = [ @{$value->{version} } ];
 	    $self->{qv} = 1 if $value->{qv};
@@ -46,9 +48,14 @@ sub new
 	    return $self;
 	}
 
-	require POSIX;
-	my $currlocale = POSIX::setlocale(&POSIX::LC_ALL);
-	my $radix_comma = ( POSIX::localeconv()->{decimal_point} eq ',' );
+	my $currlocale = setlocale(LC_ALL);
+
+	# if the current locale uses commas for decimal points, we
+	# just replace commas with decimal places, rather than changing
+	# locales
+	if ( localeconv()->{decimal_point} eq ',' ) {
+	    $value =~ tr/,/./;
+	}
 
 	if ( not defined $value or $value =~ /^undef$/ ) {
 	    # RT #19517 - special case for undef comparison
@@ -65,18 +72,11 @@ sub new
 	$value = _un_vstring($value);
 
 	# exponential notation
-	if ( $value =~ /\d+.?\d*e-?\d+/ ) {
+	if ( $value =~ /\d+.?\d*e[-+]?\d+/ ) {
 	    $value = sprintf("%.9f",$value);
-	    $value =~ s/(0+)$//;
+	    $value =~ s/(0+)$//; # trim trailing zeros
 	}
 	
-	# if the original locale used commas for decimal points, we
-	# just replace commas with decimal places, rather than changing
-	# locales
-	if ( $radix_comma ) {
-	    $value =~ tr/,/./;
-	}
-
 	# This is not very efficient, but it is morally equivalent
 	# to the XS code (as that is the reference implementation).
 	# See vutil/vutil.c for details
@@ -100,7 +100,7 @@ sub new
 	$start = $last = $pos = $s;
 		
 	# pre-scan the input string to check for decimals/underbars
-	while ( substr($value,$pos,1) =~ /[._\d]/ ) {
+	while ( substr($value,$pos,1) =~ /[._\d,]/ ) {
 	    if ( substr($value,$pos,1) eq '.' ) {
 		if ($alpha) {
 		    Carp::croak("Invalid version format ".
@@ -118,6 +118,12 @@ sub new
 		$alpha = 1;
 		$width = $pos - $last - 1; # natural width of sub-version
 	    }
+	    elsif ( substr($value,$pos,1) eq ','
+		    and substr($value,$pos+1,1) =~ /[0-9]/ ) {
+		# looks like an unhandled locale
+		$saw_period++;
+		$last = $pos;
+	    }
 	    $pos++;
 	}
 
@@ -220,6 +226,10 @@ sub new
 		    && substr($value,$pos+1,1) =~ /\d/ ) {
 		    $s = ++$pos;
 		}
+		elsif ( substr($value,$pos,1) eq ',' 
+		    && substr($value,$pos+1,1) =~ /\d/ ) {
+		    $s = ++$pos;
+		}
 		elsif ( substr($value,$pos,1) =~ /\d/ ) {
 		    $s = $pos;
 		}
@@ -271,6 +281,8 @@ sub new
 	return ($self);
 }
 
+*parse = \&new;
+
 sub numify 
 {
     my ($self) = @_;
@@ -355,7 +367,11 @@ sub stringify
 	require Carp;
 	Carp::croak("Invalid version object");
     }
-    return $self->{original};
+    return exists $self->{original} 
+    	? $self->{original} 
+	: exists $self->{qv} 
+	    ? $self->normal
+	    : $self->numify;
 }
 
 sub vcmp
@@ -443,14 +459,21 @@ sub is_alpha {
 }
 
 sub qv {
-    my ($value) = @_;
+    my $value = shift;
+    my $class = 'version';
+    if (@_) {
+	$class = ref($value) || $value;
+	$value = shift;
+    }
 
     $value = _un_vstring($value);
     $value = 'v'.$value unless $value =~ /(^v|\d+\.\d+\.\d)/;
-    my $version = version->new($value); # always use base class
+    my $version = $class->new($value);
     return $version;
 }
 
+*declare = \&qv;
+
 sub is_qv {
     my ($self) = @_;
     return (exists $self->{qv});
@@ -483,68 +506,62 @@ sub _un_vstring {
     return $value;
 }
 
-# Thanks to Yitzchak Scott-Thoennes for this mode of operation
-{
-    local $^W;
-    *UNIVERSAL::VERSION = sub {
-	my ($obj, $req) = @_;
-	my $class = ref($obj) || $obj;
+sub _VERSION {
+    my ($obj, $req) = @_;
+    my $class = ref($obj) || $obj;
 
-	no strict 'refs';
-	eval "require $class" unless %{"$class\::"}; # already existing
-	return undef if $@ =~ /Can't locate/ and not defined $req;
-	
-	if ( not %{"$class\::"} and $] >= 5.008) { # file but no package
+    no strict 'refs';
+    if ( exists $INC{"$class.pm"} and not %{"$class\::"} and $] >= 5.008) {
+	 # file but no package
+	require Carp;
+	Carp::croak( "$class defines neither package nor VERSION"
+	    ."--version check failed");
+    }
+
+    my $version = eval "\$$class\::VERSION";
+    if ( defined $version ) {
+	local $^W if $] <= 5.008;
+	$version = version::vpp->new($version);
+    }
+
+    if ( defined $req ) {
+	unless ( defined $version ) {
 	    require Carp;
-	    Carp::croak( "$class defines neither package nor VERSION"
-		."--version check failed");
-	}
-	
-	my $version = eval "\$$class\::VERSION";
-	if ( defined $version ) {
-	    local $^W if $] <= 5.008;
-	    $version = version::vpp->new($version);
-	}
+	    my $msg =  $] < 5.006 
+	    ? "$class version $req required--this is only version "
+	    : "$class does not define \$$class\::VERSION"
+	      ."--version check failed";
 
-	if ( defined $req ) {
-	    unless ( defined $version ) {
-		require Carp;
-		my $msg =  $] < 5.006 
-		? "$class version $req required--this is only version "
-		: "$class does not define \$$class\::VERSION"
-		  ."--version check failed";
-
-		if ( $ENV{VERSION_DEBUG} ) {
-		    Carp::confess($msg);
-		}
-		else {
-		    Carp::croak($msg);
-		}
+	    if ( $ENV{VERSION_DEBUG} ) {
+		Carp::confess($msg);
 	    }
-
-	    $req = version::vpp->new($req);
-
-	    if ( $req > $version ) {
-		require Carp;
-		if ( $req->is_qv ) {
-		    Carp::croak( 
-			sprintf ("%s version %s required--".
-			    "this is only version %s", $class,
-			    $req->normal, $version->normal)
-		    );
-		}
-		else {
-		    Carp::croak( 
-			sprintf ("%s version %s required--".
-			    "this is only version %s", $class,
-			    $req->stringify, $version->stringify)
-		    );
-		}
+	    else {
+		Carp::croak($msg);
 	    }
 	}
 
-	return defined $version ? $version->stringify : undef;
-    };
+	$req = version::vpp->new($req);
+
+	if ( $req > $version ) {
+	    require Carp;
+	    if ( $req->is_qv ) {
+		Carp::croak( 
+		    sprintf ("%s version %s required--".
+			"this is only version %s", $class,
+			$req->normal, $version->normal)
+		);
+	    }
+	    else {
+		Carp::croak( 
+		    sprintf ("%s version %s required--".
+			"this is only version %s", $class,
+			$req->stringify, $version->stringify)
+		);
+	    }
+	}
+    }
+
+    return defined $version ? $version->stringify : undef;
 }
 
 1; #this line is important and will help the module return a true value
